//  Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package hivemind.test.ant;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.hivemind.Resource;
import org.apache.hivemind.sdl.SDLDocumentAdaptor;
import org.apache.hivemind.test.HiveMindTestCase;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.helpers.AttributesImpl;

/**
 * Tests for the {@link org.apache.hivemind.ant.ParseSDLIntoDocument} class.
 *
 * @author Howard Lewis Ship
 */
public class TestSDLDocumentAdaptor extends HiveMindTestCase
{
    public void testMinimalDocument() throws Exception
    {
        Document d = createDocument();
        SDLDocumentAdaptor adaptor = new SDLDocumentAdaptor(d);

        AttributesImpl a = new AttributesImpl();

        adaptor.startElement(null, "module", null, a);
        adaptor.endElement(null, "module", null);

        Element e = d.getDocumentElement();

        assertEquals("module", e.getTagName());
    }

    public void testNodeContent() throws Exception
    {
        Document d = createDocument();
        SDLDocumentAdaptor adaptor = new SDLDocumentAdaptor(d);

        AttributesImpl a = new AttributesImpl();

        adaptor.startElement(null, "module", null, a);

        adaptor.characters("text".toCharArray(), 0, 4);

        a = new AttributesImpl();

        adaptor.startElement(null, "fred", null, a);
        adaptor.endElement(null, "fred", null);
        adaptor.endElement(null, "module", null);

        Element e = d.getDocumentElement();
        assertEquals("module", e.getTagName());

        CDATASection cd = (CDATASection) e.getFirstChild();

        assertEquals("text", cd.getData());

        Element fred = (Element) cd.getNextSibling();

        assertEquals("fred", fred.getTagName());
    }

    public void testAttributes() throws Exception
    {
        Document d = createDocument();
        SDLDocumentAdaptor adaptor = new SDLDocumentAdaptor(d);

        AttributesImpl a = new AttributesImpl();
        a.addAttribute(null, "id", null, "CDATA", "foo.bar.baz");

        adaptor.startElement(null, "module", null, a);
        adaptor.endElement(null, "module", null);

        Element e = d.getDocumentElement();
        assertEquals("module", e.getTagName());

        assertEquals("foo.bar.baz", e.getAttribute("id"));

        assertEquals(1, e.getAttributes().getLength());
    }

    public void testParse() throws Exception
    {
        Document d = createDocument();
        SDLDocumentAdaptor adaptor = new SDLDocumentAdaptor(d);

        Resource r = getResource("Sample.sdl");

        adaptor.parse(r);

        Element e = d.getDocumentElement();
        assertEquals("module", e.getTagName());

        assertEquals("hivemind", e.getAttribute("id"));
        assertEquals("1.0.0", e.getAttribute("version"));

        assertEquals(2, e.getAttributes().getLength());
    }

    private Document createDocument() throws Exception
    {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        DocumentBuilder builder = factory.newDocumentBuilder();

        return builder.newDocument();
    }

}
