//  Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package hivemind.test.services.impl;

import java.lang.reflect.Modifier;

import org.apache.hivemind.service.BodyBuilder;
import org.apache.hivemind.service.ClassFab;
import org.apache.hivemind.service.MethodSignature;
import org.apache.hivemind.service.impl.AbstractServiceInterceptorFactory;

/**
 * Simple factory that uses dynamic proxies to count the number of times
 * methods in interfaces it has enhanced have been executed.
 *
 * @author Howard Lewis Ship
 */
public class CountFactory extends AbstractServiceInterceptorFactory
{
    private static int _count = 0;

    public static int getCount()
    {
        return _count;
    }

    public static void reset()
    {
        _count = 0;
    }

    public static void incrementCount()
    {
        _count++;
    }

    protected void addServiceMethodImplementation(ClassFab classFab, MethodSignature sig)
    {
        BodyBuilder builder = new BodyBuilder();

        builder.begin();
        builder.addln("hivemind.test.services.impl.CountFactory#incrementCount();");

        builder.add("return ($r) _inner.");
        builder.add(sig.getName());
        builder.add("($$);");

        builder.end();

        classFab.addMethod(Modifier.PUBLIC, sig, builder.toString());
    }

}
