//  Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package hivemind.test.servlet;

import javax.servlet.Filter;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import org.apache.hivemind.service.ThreadCleanupListener;
import org.apache.hivemind.service.ThreadEventNotifier;
import org.apache.hivemind.servlet.HiveMindFilter;
import org.apache.hivemind.test.HiveMindTestCase;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.HiveMind;
import org.apache.hivemind.Registry;

public class TestHiveMindFilter extends HiveMindTestCase
{
    private static class Listener implements ThreadCleanupListener
    {
        private boolean _cleanup;

        public void threadDidCleanup()
        {
            _cleanup = true;
        }

        public boolean getCleanup()
        {
            return _cleanup;
        }

    }

    public void testBasic() throws Exception
    {
        ServletContext servletContext = new MockServletContext();
        FilterConfig filterConfig = new MockFilterConfig(servletContext);

        Filter f = new HiveMindFilter();

        f.init(filterConfig);

        Registry r = (Registry) servletContext.getAttribute(HiveMindFilter.CONTEXT_KEY);

        assertNotNull(r);

        ThreadEventNotifier t =
            (ThreadEventNotifier) r.getService(
                HiveMind.THREAD_EVENT_NOTIFIER_SERVICE,
                ThreadEventNotifier.class);

        Listener l = new Listener();

        t.addThreadCleanupListener(l);

        HttpServletRequest request = new MockRequest();

        f.doFilter(request, new MockResponse(), new MockFilterChain());

        assertSame(r, HiveMindFilter.getRegistry(request));

        assertEquals(true, l.getCleanup());

        f.destroy();

        try
        {
            t.addThreadCleanupListener(null);
            unreachable();
        }
        catch (ApplicationRuntimeException ex)
        {
            assertExceptionSubstring(ex, "The HiveMind Registry has been shutdown.");
        }
    }

    public void testExceptionInInit() throws Exception
    {
        Filter f = new HiveMindFilter();

        interceptLogging(HiveMindFilter.class.getName());

        f.init(null);

        // It's a NPE that gets thrown.

        assertLoggedMessage("null");

        f.doFilter(new MockRequest(), new MockResponse(), new MockFilterChain());

        f.destroy();
    }

    public void testDestroyWithoutRepository()
    {
        Filter f = new HiveMindFilter();

        f.destroy();
    }

    public void testFilterWithoutRepository() throws Exception
    {
        Filter f = new HiveMindFilter();

        interceptLogging(HiveMindFilter.class.getName());

        f.doFilter(new MockRequest(), new MockResponse(), new MockFilterChain());

        assertLoggedMessage("Unable to cleanup current thread");
    }

}
